extern crate linearhash;

use linearhash::LinHash;
use std::time::Instant;
use std::fs;
use linearhash::util::*;

#[allow(dead_code)]
fn measure_perf(num_iters: i32) {
    // in each iteration, insert a larger number of records to see how
    // `insert` and `lookup` performs. `insert` should be O(n) and
    // `lookup` should be O(1).
    for i in 1..num_iters {
        fs::remove_file("tmp/measure_perf").ok();
        let time_insert_start = Instant::now();
        let mut h2 = LinHash::open("tmp/measure_perf", 16, 16);
        for k in 0..(10000*i) {
            h2.put(&linearhash::util::i32_to_bytearray(k),
                   &linearhash::util::i32_to_bytearray(k+1));
        }
        let time_insert_end = Instant::now();
        println!("[insert] 10000 records {:?}", time_insert_end.duration_since(time_insert_start));

        let time_get_start = Instant::now();
        for k in 0..10000 {
            // assert_eq!(h2.get(&linearhash::util::i32_to_bytearray(k)),
            //            Some(linearhash::util::i32_to_bytearray(k+1).to_vec()));
            h2.get(&linearhash::util::i32_to_bytearray(k));
        }
        let time_get_end = Instant::now();
        println!("[get] 10000 records {:?}", time_get_end.duration_since(time_get_start));

        let all_end = Instant::now();
        println!("[insert+get] 10000 records {:?}", all_end.duration_since(time_insert_start));
        h2.close();
        // fs::remove_file("tmp/measure_perf");
    }

}

fn main() {

    measure_perf(2);

    // fs::remove_file("tmp/main_tests").ok();
    // // let mut cache: LruCache<K, V> = LruCache::new(1000);
    // let mut h = LinHash::open("tmp/main_tests", 32, 4);
    // h.put(b"Spin", &i32_to_bytearray(9));
    // h.put(b"Axis", &i32_to_bytearray(6));
    // h.put(b"foo", &[14]);
    // h.put(b"bar", &[15]);
    // h.put(b"linear", &[16]);
    // h.put(b"hashing", &[17]);
    // h.put(b"disk", &[18]);
    // h.put(b"space", &[19]);
    // h.put(b"random", &[20]);
    // h.put(b"keys", &[21]);
    // h.put(b"ishaan", &[22]);
    // h.put(b"linearhash", &[21]);
    // h.put(b"rust", &[21]);
    // h.put(b"3:30", &[21]);
    // h.put(b"xinu", &[21]);
    // h.put(b"linearhash1", &[21]);
    // h.put(b"rust1", &[22]);
    // h.put(b"rust2", &[51]);
    // h.put(b"rust3", &[52]);
    // h.put(b"rust4", &[53]);
    // h.put(b"rust5", &[54]);
    //
    // h.update(b"rust1", &[99]);
    // h.put(b"xinu3", &[24]);
    // h.close();
    //
    // println!("{:?}", h.get("rust3".as_bytes()));
}
